/***********************************************************************
*                    SEGGER Microcontroller GmbH                       *
*                        The Embedded Experts                          *
************************************************************************
*                                                                      *
*                  (c) SEGGER Microcontroller GmbH                     *
*                        All rights reserved                           *
*                          www.segger.com                              *
*                                                                      *
************************************************************************
*                                                                      *
************************************************************************
*                                                                      *
*                                                                      *
*  Licensing terms                                                     *
*                                                                      *
* Redistribution and use in source and binary forms, with or without   *
* modification, are permitted provided that the following conditions   *
* are met:                                                             *
*                                                                      *
* 1. Redistributions of source code must retain the above copyright    *
* notice, this list of conditions and the following disclaimer.        *
*                                                                      *
* 2. Redistributions in binary form must reproduce the above           *
* copyright notice, this list of conditions and the following          *
* disclaimer in the documentation and/or other materials provided      *
* with the distribution.                                               *
*                                                                      *
*                                                                      *
* THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDER "AS IS" AND ANY        *
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    *
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR   *
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDER BE        *
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,     *
* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,             *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   *
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  *
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         *
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH.    *
* DAMAGE.                                                              *
*                                                                      *
************************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : RAMCODE_Intern.h
Purpose : Use separate header file for RAMCODE includes to not being forced to include GLOBAL.h and other headers as well.
          RAMCODE prototypes etc. usually do not change that often.
*/

#ifndef RAMCODE_INTERN_H
#define RAMCODE_INTERN_H

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

//
// May be already defined if FlashOS.h is also included by algo
//
#ifndef U8
  #define U8  unsigned char
#endif
#ifndef U16
  #define U16 unsigned short
#endif
#ifndef U32
  #define U32 unsigned long
#endif
#ifndef U64
  #ifdef _MSC_VER
    #define U64   unsigned __int64
  #else
    #define U64   unsigned long long
  #endif
#endif
#ifndef I8
  #define I8  signed char
#endif
#ifndef I16
  #define I16 signed short
#endif
#ifndef I32
  #define I32 signed long
#endif
#ifndef I64
  #ifdef _MSC_VER
    #define I64   signed __int64
  #else
    #define I64   signed long long
  #endif
#endif

#ifndef   COUNTOF
  #define COUNTOF(a)    (int)(sizeof(a)/sizeof(a[0]))
#endif
#ifndef   MIN
  #define MIN(a, b)     (((a) < (b)) ? (a) : (b))
#endif
#ifndef   MAX
  #define MAX(a, b)     (((a) > (b)) ? (a) : (b))
#endif

//
// Special results which are not really errors but for special handling on buggy chips
//
#define RAMCODE_RESULT_ERROR_PROGRAM_NO_AUTO_BLOCK_VERIFY_ALLOWED (0x40000000)
#define RAMCODE_RESULT_ERROR_ERASE_NO_AUTO_BLANK_CHECK_ALLOWED    (0x40000001)
//
// RAMCode SPIFI support flags (Returned on GetDevDesc())
//
#define RAMCODE_SPIFI_SUPPORT_FLAG_MINALIGN_MASK        (0xFFuL << 0)
#define RAMCODE_SPIFI_SUPPORT_FLAG_MINALIGN_BITPOS      (0)
#define RAMCODE_SPIFI_SUPPORT_FLAG_MULTI_SECTOR_ERASE   (1uL << 8)
#define RAMCODE_SPIFI_SUPPORT_FLAG_MULTI_SECTOR_PROG    (1uL << 9)
#define RAMCODE_SPIFI_SUPPORT_FLAG_ERASE_CHIP           (1uL << 10)
#define RAMCODE_SPIFI_SUPPORT_FLAG_AUTO_ERASE_ON_PROG   (1uL << 11)
//
// RAMCode generic support flags (compiled into generic part)
//
#define RAMCODE_STATIC_SUPPORT_DISABLE_1ST_LVL_VERIFY  (1 << 0)  // Supports RAMCODE_PROG_FLAG_DISABLE_1ST_LVL_VERIFY
// [31:1] free for future use. Use positive logic (1 == supported) for new flags
//
// RAMCode flags for generic part that may be passed on <Program> as part of <NumBytes>
//
#define RAMCODE_PROG_FLAG_DISABLE_1ST_LVL_VERIFY        (1 << 24)
//#define RAMCODE_PROG_FLAG_RESERVED                    (1 << 25)
//#define RAMCODE_PROG_FLAG_RESERVED                    (1 << 26)
//#define RAMCODE_PROG_FLAG_RESERVED                    (1 << 27)
//#define RAMCODE_PROG_FLAG_RESERVED                    (1 << 28)
//#define RAMCODE_PROG_FLAG_RESERVED                    (1 << 29)
//#define RAMCODE_PROG_FLAG_RESERVED                    (1 << 30)
//#define RAMCODE_PROG_FLAG_RESERVED                    (1 << 31)

/*********************************************************************
*
*       Types
*
**********************************************************************
*/

typedef struct {
  U32     Offset;                        // Usually 0. Only set in case current sectors start with a gap before them.
  U32     NumSectors;
  U32     SectorSize;
} RAMCODE_FLASH_BLOCK;

typedef struct {
  U32   Flags;                        // 0:7 MinAlignShift, 8:8 MultiSectorErase, 9:9 MultiSectorProg, 10:10 EraseChip
  U32   NumBlocks;                    // Number of BlockInfo elements
  RAMCODE_FLASH_BLOCK BlockInfo;      // This is the first block of an array of flash blocks
} RAMCODE_SPIFI_DEVICE_DESC;

typedef union {
  struct {
    void* pSrc;
    void* pDest;
    U32   NumHWords;
    U32   WritePara0;
    U32   WritePara1;
    U32   WritePara2;
  } In;
  struct {
    void* FailAddr;
  } Out;
} IO_PARA;

/*********************************************************************
*
*       Function declarations
*
**********************************************************************
*/

int  GetDeviceDesc     (void* pBuffer, U32 BufferSize);
int  Program           (IO_PARA* p);
int  Erase             (U32 NumSectors, U32 SectorAddr, U32 ErasePara1,  U32 ErasePara2);
void FeedWatchdog      (void);
int  Prepare           (void* pRestoreInfo);
int  Restore           (void* pRestoreInfo);

#endif // RAMCODE_INTERN_H

/****** End Of File *************************************************/
